<?php

namespace Modules\Cotacao\Modules\Pedido\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\Table;
use PHPExcel_Cell;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;

class ImportacaoRFPController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    private $importRequest;

    protected $service;
    protected $crudService;
    protected $parameterBag;
    protected $environment;
    protected $connection;

    public function __construct($crudService, $parameterBag, $environment, $entityManager)
    {
        parent::__construct($crudService, $parameterBag);
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function importarRFP(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $this->importRequest = $request;
            $table = $this->xlsRead($row);
            $response->addDataSet(new DataSet('data', $table));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function checkItens(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            if (isset($row) && isset($row['NRRFPMAIN'])) {
                $this->execCheckImportProcedure($row);
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    private function execCheckImportProcedure($row)
    {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'TEKNISA_RFP.CHECK_IMPORT');
            $procedure->addParam(new Param('NRRFPMAIN', Param::PARAM_INPUT, $row['NRRFPMAIN'], Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function readFile($row)
    {
        try {
            $file = $row['FILE'];
            $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);

            if (!$xlsFile) {
                throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
            }

            return $xlsFile->getActiveSheet();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function limpaCPF_CNPJ($valor)
    {
        if (!isset($valor)) {
            return null;
        }
        $valor = trim($valor);
        $valor = str_replace(".", "", $valor);
        $valor = str_replace(",", "", $valor);
        $valor = str_replace("-", "", $valor);
        $valor = str_replace("/", "", $valor);
        return $valor;
    }

    private function checkRegiaoValidate(&$regions, &$UFServed, &$regionsException)
    {
        try {
            $regionsLenght = count(array_filter($regions));
            $UFLenght = count(array_filter($UFServed));
            $regionsExceptionLenght = count(array_filter($regionsException));
            $contentLenght = 0;

            if ($regionsExceptionLenght && !$UFLenght) {
                throw new \Exception("SPREADSHEET_CONFLICT");
            }
            if ($regionsLenght == 0 && $UFLenght == 0) {
                throw new \Exception("REGION_OR_UF_NULL");
            }
            if ($regionsLenght && $regionsExceptionLenght) {
                foreach ($regions as $region) {
                    if ($region != null) {
                        if (in_array($region, $regionsException)) {
                            $contentLenght++;
                        }
                    }
                }
                if ($contentLenght > 0) {
                    throw new \Exception("REGION_EXCEPTION");
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insertHeader($worksheet, &$row)
    {
        try {
            $regions = array();
            $UFServed = array();
            $regionsException = array();
            $cnpjs = array();

            $this->setCellValues($worksheet, $row, $regions, $UFServed, $regionsException, $cnpjs);

            if (in_array(null, $row, true)) {
                throw new \Exception("CHECK_SPREADSHEET");
            }

            $this->saveRfpMain($row);

            $regionsLenght = count(array_filter($regions));
            $UFLenght = count(array_filter($UFServed));

            if ($UFLenght && $regionsLenght) {
                $UFRegions = $this->getFederativeUnit(implode("_", $UFServed), implode("_", $regionsException));
                if (isset($UFRegions) && count($UFRegions)) {
                    $unitRegion = [];
                    foreach ($UFRegions as $federativeUnit) {
                        array_push($unitRegion, $federativeUnit['CDREGIAO']);
                    }
                    $arrayUnit = array_merge($regions, $unitRegion);
                    $unitFormated = array_unique(array_map(function ($row) {
                        return $row;
                    }, $arrayUnit));
                    foreach ($unitFormated as $regiao) {
                        if ($regiao != null) {
                            $this->saveRfpRegiao($row, $regiao);
                        }
                    }
                }
            } else if ($UFLenght) {
                $UFRegions = $this->getFederativeUnit(implode("_", $UFServed), implode("_", $regionsException));
                if (isset($UFRegions) && count($UFRegions)) {
                    foreach ($UFRegions as $UFRegion) {
                        $this->saveRfpRegiao($row, $UFRegion['CDREGIAO']);
                    }
                }
            } else if ($regionsLenght) {
                foreach ($regions as $region) {
                    if (isset($region) && !empty($region)) {
                        $this->saveRfpRegiao($row, $region);
                    }
                }
            }

            $cnpjLenght = count(array_filter($cnpjs));

            if ($cnpjLenght) {
                foreach ($cnpjs as $cnpj) {
                    if (isset($cnpj) && !empty($cnpj)) {
                        $this->saveRfpFornecedor($row, $cnpj);
                    }
                }
            }

            return $row;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function generalSave($row, $dataSourceName)
    {
        try {
            $this->dataSourceNameToSave = $dataSourceName;
            $requestToSave = new DTO\Request\Row($row, $this->importRequest->getMethod(), $this->importRequest->getRoutePath(), $this->importRequest->getUserId());
            $newResponse = new DTO\Response();
            parent::save($requestToSave, $newResponse);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function saveRfpMain(&$row)
    {
        try {
            $row['__is_new'] = true;
            $row['NRRFPMAIN'] = parent::sequenceGenerator("SEQ_RFPMAIN_NRRFPMAIN");
            $this->generalSave($row, "rfpmain");
            return $row['NRRFPMAIN'];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function saveRfpRegiao(&$row, $codregiao)
    {
        try {
            $row['__is_new'] = true;
            $row['CODREGIAO'] = $codregiao;
            $row['NRRFPREGIAO'] = parent::sequenceGenerator("SEQ_RFPREGIAO_NRRFPREGIAO");
            $this->generalSave($row, "rfpregiao");
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveRfpFornecedor(&$row, $insJurForn)
    {
        try {
            $row['__is_new'] = true;
            $row['NRINSJURFRFP'] = $insJurForn;
            $row['NRRFPFORNECEDOR'] = parent::sequenceGenerator("SEQ_RFPFORNECEDOR_NRRFPFORN");
            $this->generalSave($row, "rfpfornecedor");
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveRfpProdmarca($row)
    {
        try {
            $row['__is_new'] = true;
            $row['NRRFPPRODMARCA'] = parent::sequenceGenerator("SEQ_RFPPRODMARCA_NRRFPPRDMARCA");
            $this->generalSave($row, "rfpprodmarca");
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getYear($date, $format = 'd/m/Y')
    {
        $date = \DateTime::createFromFormat($format, $date);
        return $date->format("Y");
    }

    private function validateYear($date)
    {
        if ($this->getYear($date) < 1901) {
            throw new \Exception("INVALID_VALIDITY");
        }
    }

    private function checkValidade(&$row, $dtInicial, $dtFinal)
    {
        try {
            if (!isset($dtInicial) || !isset($dtFinal)) {
                throw new \Exception("VALIDADE_NAO_INFORMADA");
            }

            $dtInicial = (gmdate('d/m/Y', ((int)$dtInicial - 25569) * 86400));
            $dtFinal = (gmdate('d/m/Y', ((int)$dtFinal - 25569) * 86400));

            $this->validateYear($dtInicial);
            $this->validateYear($dtFinal);

            if ($dtInicial > $dtFinal) {
                throw new \Exception("VALIDITY_CHECK");
            }

            $row["DTVALIDADEINICIAL"] = $dtInicial;
            $row["DTVALIDADEFINAL"] = $dtFinal;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function setCellValues($worksheet, &$row, &$regions, &$UFServed, &$regionsException, &$cnpjs)
    {
        //Comprador
        $comprador = trim($worksheet->getCell('B6')->getValue()); //Codigo comprador valor
        if (!isset($comprador)) {
            throw new \Exception("COMPRADOR_NAO_INFORMADO");
        }
        $row["COMPRADOR"] = str_pad($comprador, 12, '0', STR_PAD_LEFT);
        $this->checkComprador($row["COMPRADOR"]);

        //Validade
        $dtInicial = $worksheet->getCell('D3')->getValue();
        $dtFinal = $worksheet->getCell('D4')->getValue();
        $this->checkValidade($row, $dtInicial, $dtFinal);

        //Fornecedor
        $cnpjs = [
            "0" => $this->limpaCPF_CNPJ($worksheet->getCell('D8')->getValue()), //CNPJ 1 valor
            "1" => $this->limpaCPF_CNPJ($worksheet->getCell('G8')->getValue()), //CNPJ 2 valor
            "2" => $this->limpaCPF_CNPJ($worksheet->getCell('J8')->getValue()), //CNPJ 3 valor
            "3" => $this->limpaCPF_CNPJ($worksheet->getCell('M8')->getValue()), //CNPJ 4 valor
            "4" => $this->limpaCPF_CNPJ($worksheet->getCell('P8')->getValue()), //CNPJ 5 valor
            "5" => $this->limpaCPF_CNPJ($worksheet->getCell('D9')->getValue()), //CNPJ 6 valor
            "6" => $this->limpaCPF_CNPJ($worksheet->getCell('G9')->getValue()), //CNPJ 7 valor
            "7" => $this->limpaCPF_CNPJ($worksheet->getCell('J9')->getValue()), //CNPJ 8 valor
            "8" => $this->limpaCPF_CNPJ($worksheet->getCell('M9')->getValue()), //CNPJ 9 valor
            "9" => $this->limpaCPF_CNPJ($worksheet->getCell('P9')->getValue()), //CNPJ 10 valor
        ];

        $cnpjLenght = count(array_filter($cnpjs));
        if (!$cnpjLenght) {
            throw new \Exception("FORNECEDOR_NAO_INFORMADO");
        }
        foreach ($cnpjs as $cnpj) {
            if ($cnpj != null) {
                $this->checkFornecedor($cnpj);
            }
        }

        //Regiões
        $regions = [
            "0"  => trim($worksheet->getCell('E3')->getValue()), //Regiao 1
            "1"  => trim($worksheet->getCell('E4')->getValue()), //Regiao 2
            "2"  => trim($worksheet->getCell('E5')->getValue()), //Regiao 3
            "3"  => trim($worksheet->getCell('E6')->getValue()), //Regiao 4
            "4"  => trim($worksheet->getCell('F3')->getValue()), //Regiao 5
            "5"  => trim($worksheet->getCell('F4')->getValue()), //Regiao 6
            "6"  => trim($worksheet->getCell('F5')->getValue()), //Regiao 7
            "7"  => trim($worksheet->getCell('F6')->getValue()), //Regiao 8
            "8"  => trim($worksheet->getCell('O3')->getValue()), //Regiao 9
            "9"  => trim($worksheet->getCell('O4')->getValue()), //Regiao 10
            "10" => trim($worksheet->getCell('O5')->getValue()), //Regiao 11
            "11" => trim($worksheet->getCell('O6')->getValue()), //Regiao 12
            "12" => trim($worksheet->getCell('P3')->getValue()), //Regiao 13
            "13" => trim($worksheet->getCell('P4')->getValue()), //Regiao 14
            "14" => trim($worksheet->getCell('P5')->getValue()), //Regiao 15
            "15" => trim($worksheet->getCell('P6')->getValue()), //Regiao 16
            "16" => trim($worksheet->getCell('Q3')->getValue()), //Regiao 17
            "17" => trim($worksheet->getCell('Q4')->getValue()), //Regiao 18
            "18" => trim($worksheet->getCell('Q5')->getValue()), //Regiao 19
            "19" => trim($worksheet->getCell('Q6')->getValue()), //Regiao 20
        ];

        $regions = array_map(function ($value) {
            return isset($value) && !empty($value) ? str_pad($value, 3, '0', STR_PAD_LEFT) : null;
        }, $regions);

        //UFs
        $UFServed = [
            "0"  => trim($worksheet->getCell('H2')->getValue()), //UF Atendida 1
            "1"  => trim($worksheet->getCell('H3')->getValue()), //UF Atendida 2
            "2"  => trim($worksheet->getCell('H4')->getValue()), //UF Atendida 3
            "3"  => trim($worksheet->getCell('H5')->getValue()), //UF Atendida 4
            "4"  => trim($worksheet->getCell('H6')->getValue()), //UF Atendida 5
            "5"  => trim($worksheet->getCell('I2')->getValue()), //UF Atendida 6
            "6"  => trim($worksheet->getCell('I3')->getValue()), //UF Atendida 7
            "7"  => trim($worksheet->getCell('I4')->getValue()), //UF Atendida 8
            "8"  => trim($worksheet->getCell('I5')->getValue()), //UF Atendida 9
            "9"  => trim($worksheet->getCell('I6')->getValue()), //UF Atendida 10
        ];

        $UFServed = array_map(function ($value) {
            return isset($value) && !empty($value) ? str_pad($value, 2, '0', STR_PAD_LEFT) : null;
        }, $UFServed);

        //Exceções regiões
        $regionsException = [
            "0"  => trim($worksheet->getCell('J3')->getValue()), //Exceção Região 1
            "1"  => trim($worksheet->getCell('J4')->getValue()), //Exceção Região 2
            "2"  => trim($worksheet->getCell('J5')->getValue()), //Exceção Região 3
            "3"  => trim($worksheet->getCell('J6')->getValue()), //Exceção Região 4
        ];

        $regionsException = array_map(function ($value) {
            return isset($value) && !empty($value) ? str_pad($value, 3, '0', STR_PAD_LEFT) : null;
        }, $regionsException);

        $this->checkRegiaoValidate($regions, $UFServed, $regionsException);

        foreach ($regions as $regiao) {
            if ($regiao != null) {
                $this->checkRegiao($regiao);
            }
        }
        foreach ($UFServed as $unidadeFederativa) {
            if ($unidadeFederativa != null) {
                $this->checkUnidadeFederativa($unidadeFederativa);
            }
        }
    }

    private function generalVerification($dataSourceName, $exceptionMsg, $conditions)
    {
        try {
            $this->dataSourceName = $dataSourceName;
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            foreach ($conditions as $name => $value) {
                $filterCriteria->addCondition($name, $value);
            }
            $newRequest = new Filter($filterCriteria, $this->importRequest->getMethod(), $this->importRequest->getRoutePath(), $this->importRequest->getUserId());
            $newResponse = new DTO\Response();

            parent::find($newRequest, $newResponse);
            $data = $newResponse->getDataSets();

            $resultData = array();

            if (is_array($data) && count($data)) {
                $data = end($data);
                if ($data->getDataSourceName() === $this->dataSourceName) {
                    $resultData = $data->getRows();
                }
            }

            if (!count($resultData)) {
                throw new \Exception($exceptionMsg);
            }

            return $resultData;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function checkFornecedor($cnpjs)
    {
        try {
            $conditions = array("NRINSJURFORN" => $cnpjs);
            $this->generalVerification("fornecedor", "FORNECEDOR_NOT_REGISTERED", $conditions);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function checkComprador($comprador)
    {
        try {
            $conditions = array("CDOPERADOR" => $comprador);
            $this->generalVerification("operador", "COMPRADOR_N_CADASTRADO", $conditions);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function checkRegiao($regiao)
    {
        try {
            $conditions = array("CDREGIAO" => $regiao);
            $this->generalVerification("regiao", "REGION_NOT_REGISTERED", $conditions);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function checkUnidadeFederativa($estado)
    {
        try {
            $conditions = array("SGESTADO" => $estado);
            $this->generalVerification("estado", "UF_NOT_REGISTERED", $conditions);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getFederativeUnit($uf, $regiao)
    {
        try {
            $conditions = array(
                "CDPAIS" => "0055",
                "SGESTADO" => $uf,
                "CDREGIAO" => $regiao,
            );
            return $this->generalVerification("federativeUnit", "UF_NOT_REGISTERED", $conditions);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function columnNames()
    {
        return array("CÓDIGO", "DESCRIÇÃO", "CÓDIGO", "NOME", "PREÇO", "CÓDIGO", "NOME", "PREÇO", "CÓDIGO", "NOME", "PREÇO", "CÓDIGO", "NOME", "PREÇO", "CÓDIGO", "NOME", "PREÇO");
    }

    private function setNull(&$row)
    {
        $row["CODPRODUTO"] = null;
        $row["CODMARCA1"] = null;
        $row["NMMARCA1"] = null;
        $row["VRPRECO1"] = null;
        $row["CODMARCA2"] = null;
        $row["NMMARCA2"] = null;
        $row["VRPRECO2"] = null;
        $row["CODMARCA3"] = null;
        $row["NMMARCA3"] = null;
        $row["VRPRECO3"] = null;
        $row["CODMARCA4"] = null;
        $row["NMMARCA4"] = null;
        $row["VRPRECO4"] = null;
        $row["CODMARCA5"] = null;
        $row["NMMARCA5"] = null;
        $row["VRPRECO5"] = null;
        $row["DESCRICAO"] = null;
    }

    private function insertBody($worksheet, &$row)
    {
        $this->setNull($row);
        $columnNames = $this->columnNames();

        $minRowIndex = 14;
        if (PHP_VERSION_ID >= 70400) {
            $maxColumnIndex = 17;
        } else {
            $maxColumnIndex = 16;
        }
        $runDelete = true;

        try {
            $worksheetDimensions = $worksheet->getHighestRowAndColumn();
            $maxRowIndex = $worksheetDimensions['row'];
            $columnsReference  = self::buildColumnNameReference($worksheet, $maxColumnIndex, $columnNames);
            $tableRows = array();
            $worksheetContent = array();

            for ($index = $minRowIndex; $index <= $maxRowIndex; $index++) {
                for ($column = 0; $column <= $maxColumnIndex; $column++) {
                    if (PHP_VERSION_ID >= 70400) {
                        $columnLetterIndex = Coordinate::stringFromColumnIndex($column);
                    } else {
                        $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                    }
                    $columnName = $columnsReference[$columnLetterIndex];
                    $cellValue = trim($worksheet->getCell($columnLetterIndex . $index)->getFormattedValue());

                    if (isset($cellValue) && !empty($cellValue)) {
                        if ($columnLetterIndex == "A") {
                            $row['CODPRODUTO'] = trim(str_replace(".", "", $cellValue));
                            $worksheetContent = array_merge($worksheetContent, array($columnName => $row['CODPRODUTO']));
                        } else if ($columnLetterIndex == "B") {
                            $row['DESCRICAO'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName => $row['DESCRICAO']));
                        } else if ($columnLetterIndex == "C") {
                            $row['CODMARCA1'] = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA1" => $row['CODMARCA1']));
                        } else if ($columnLetterIndex == "D") {
                            $row['NMMARCA1'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA1" => $row['NMMARCA1']));
                        } else if ($columnLetterIndex == "E") {
                            $row['VRPRECO1'] = floatval(trim(str_replace("R$", "", str_replace(',', '', $cellValue))));
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA1" => $row['VRPRECO1']));
                        } else if ($columnLetterIndex == "F") {
                            $row['CODMARCA2'] = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA2" => $row['CODMARCA2']));
                        } else if ($columnLetterIndex == "G") {
                            $row['NMMARCA2'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA2" => $row['NMMARCA2']));
                        } else if ($columnLetterIndex == "H") {
                            $row['VRPRECO2'] = floatval(trim(str_replace("R$", "", str_replace(',', '', $cellValue))));
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA2" => $row['VRPRECO2']));
                        } else if ($columnLetterIndex == "I") {
                            $row['CODMARCA3'] = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA3" => $row['CODMARCA3']));
                        } else if ($columnLetterIndex == "J") {
                            $row['NMMARCA3'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA3" => $row['NMMARCA3']));
                        } else if ($columnLetterIndex == "K") {
                            $row['VRPRECO3'] = floatval(trim(str_replace("R$", "", str_replace(',', '', $cellValue))));
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA3" => $row['VRPRECO3']));
                        } else if ($columnLetterIndex == "L") {
                            $row['CODMARCA4'] = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA4" => $row['CODMARCA4']));
                        } else if ($columnLetterIndex == "M") {
                            $row['NMMARCA4'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA4" => $row['NMMARCA4']));
                        } else if ($columnLetterIndex == "N") {
                            $row['VRPRECO4'] = floatval(trim(str_replace("R$", "", str_replace(',', '', $cellValue))));
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA4" => $row['VRPRECO4']));
                        } else if ($columnLetterIndex == "O") {
                            $row['CODMARCA5'] = str_pad($cellValue, 5, '0', STR_PAD_LEFT);
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA5" => $row['CODMARCA5']));
                        } else if ($columnLetterIndex == "P") {
                            $row['NMMARCA5'] = $cellValue;
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA5" => $row['NMMARCA5']));
                        } else if ($columnLetterIndex == "Q") {
                            $row['VRPRECO5'] = floatval(trim(str_replace("R$", "", str_replace(',', '', $cellValue))));
                            $worksheetContent = array_merge($worksheetContent, array($columnName . "_MARCA5" => $row['VRPRECO5']));
                        }
                    }
                }

                if (isset($row['CODPRODUTO'])) {
                    $runDelete = false;
                    $worksheetRow = array(
                        'index' => $index,
                        'content' => $worksheetContent
                    );

                    $this->saveRfpProdmarca($row);
                    array_push($tableRows, $worksheetRow);
                }

                $this->setNull($row);
            }

            if ($runDelete) {
                $rowToDelete = array(
                    "__is_new" => false,
                    "NRRFPMAIN" => $row["NRRFPMAIN"],
                    "NRORG" => $this->environment->getNrOrg()
                );
                $this->deleteImportError($rowToDelete);
                throw new \Exception('PRODUCT_NOT_FOUND');
            }

            $this->execCheckImportProcedure($row);

            $table = new Table($columnsReference, $tableRows);
            $rowsFromTable = $table->getRows();

            return $rowsFromTable;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function deleteImportError($row)
    {
        try {
            $this->dataSourceNameToSave = "rfpfornecedor";
            $requestToSave = new DTO\Request\Row($row, $this->importRequest->getMethod(), $this->importRequest->getRoutePath(), $this->importRequest->getUserId());
            $newResponse = new DTO\Response();
            $this->delRfpfornecedor($requestToSave, $newResponse);

            $this->dataSourceNameToSave = "rfpregiao";
            $requestToSave = new DTO\Request\Row($row, $this->importRequest->getMethod(), $this->importRequest->getRoutePath(), $this->importRequest->getUserId());
            $newResponse = new DTO\Response();
            $this->delRfpregiao($requestToSave, $newResponse);

            $this->dataSourceNameToSave = "rfpmain";
            $requestToSave = new DTO\Request\Row($row, $this->importRequest->getMethod(), $this->importRequest->getRoutePath(), $this->importRequest->getUserId());
            $newResponse = new DTO\Response();
            $this->delRfpmain($requestToSave, $newResponse);
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function xlsRead($row)
    {
        try {
            $worksheet = $this->readFile($row);
            $file = $row['FILE'];
            $fileName = $file['name'];

            list($name, $extension) = explode(".", $fileName);
            if (strtolower($extension) === 'xls') {
                throw new \Exception("INVALID_FORMAT_XLS");
            }

            $row = [
                "STATUS" => 'PENDENTE',
                "DTIMPORTACAO" => date("d/m/Y"),
                "RESPONSAVEL" => $this->environment->getCdoperador(),
                "DSARQUIVO" => $fileName,
                "CDOPERLOGADO" => $this->environment->getCdoperador(),
                "CDFILIAL" => $this->environment->getCdFilial()
            ];

            $row = $this->insertHeader($worksheet, $row);
            $table = $this->insertBody($worksheet, $row);

            return $table;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function deleteImport(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->verifyRfpGerado($request);
            $this->delRfpitemcota($request, $response);
            $this->delRfpfornecedor($request, $response);
            $this->delRfpregiao($request, $response);
            $this->delRfpprodmarca($request, $response);
            $this->delRfpmain($request, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    private function delRfpmain($request, $response)
    {
        try {
            $this->dataSourceName = 'rfpmain';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function delRfpprodmarca($request, $response)
    {
        try {
            $this->dataSourceName = 'rfpprodmarca';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function delRfpregiao($request, $response)
    {
        try {
            $this->dataSourceName = 'rfpregiao';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function delRfpfornecedor($request, $response)
    {
        try {
            $this->dataSourceName = 'rfpfornecedor';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function delRfpitemcota($request, $response)
    {
        try {
            $this->dataSourceName = 'rfpitemcota';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function verifyRfpGerado($request)
    {
        try {
            $data = $request->getDataSet()->getRows();
            $row = end($data);
            $filterCriteria = new FilterCriteria("existsRfpmain");

            foreach ($row as $name => $value) {
                if ($name === 'NRRFPMAIN') {
                    $filterCriteria->addCondition($name, '=', $value);
                }
            }

            $filterCriteria->addCondition("STATUS", "GERADO");
            $conditions = $filterCriteria->getConditions();
            $data = $this->crudService->getFilteredDataSet("existsRfpmain", $conditions)->getRows();

            if (isset($data) && count($data)) {
                throw new \Exception("IMPOSSIVEL_EXCLUIR_GERADO");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gerarPedido(DTO\Request $request, DTO\Response $response)
    {
        $conditions = $request->getRow();

        $params = array(
            "NRRFPMAIN" => $conditions["NRRFPMAIN"],
            "IDDISMONTLIS" => $conditions["IDDISMONTLIS"]
        );
        try {
            $result = $this->execImportaRFPProcedure($params);

            if (isset($result['ERROR'])) {
                throw new \Exception($result['ERROR']);
            }

            if (isset($result['MESSAGE'])) {
                $response->addMessage(new Message($result['MESSAGE']));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function execImportaRFPProcedure(array $lista)
    {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'TEKNISA_RFP.IMPORTA_RFP');
            $procedure->addParam(new Param('NRRFPMAIN', Param::PARAM_INPUT, $lista['NRRFPMAIN'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDDISMONTLIS', Param::PARAM_INPUT, $lista['IDDISMONTLIS'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERINCL', Param::PARAM_INPUT, $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('MESSAGE', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 30));
            $procedure->addParam(new Param('ERROR', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 4000));
            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    protected static function getColumnString($column)
    {
        if (gettype($column) == 'integer')
            if (PHP_VERSION_ID >= 70400) {
                return Coordinate::stringFromColumnIndex($column);
            } else {
                return PHPExcel_Cell::stringFromColumnIndex($column);
            }
        else
            return $column;
    }

    private function buildColumnNameReference($worksheet, $maxColumn, $columnNames)
    {
        $reference = array();
        for ($column = 0; $column <= $maxColumn; $column++) {
            $columnString = self::getColumnString($column);
            $columnName = $worksheet->getCell($columnString . '13')->getValue();
            $reference[$columnString] = mb_strtoupper($columnName, 'UTF-8');
        }
        return ($reference);
    }
}
